#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/netdevice.h>
#include <linux/init.h>

static int y() {return 1;}
static void x(int a) {}
#define NW_IOADDR 1
#define NW_IRQ 1
#define FILL_ETHER_ADDRESS x
#define INIT_NW() 
#define RESET_NW() 
#define READ_INTERRUPT_CONDITION y
#define FILL_TX_NW x
#define READ_RX_NW x
#define TX_EVENT 1
#define RX_EVENT 2
#define READ_RX_LEN y

static void lxNW_Isr(int irq, void *id, struct pt_regs *regs);
static int lxNW_open(struct net_device *dev);
static int lxNW_close(struct net_device *dev);
static int lxNW_SendPacket(struct sk_buff *skb, struct net_device *dev);
static void lxNW_Receive(struct net_device *dev);

int __init lxNWprobe(struct net_device *dev)
{

	/* This function is used only in case the driver is used as a module in
	 * which case this function initialises the owner of the device
	 */
	SET_MODULE_OWNER(dev);

	/* Set the starting address for the IO access; this will be used by
	  * by the inb()/outb() family of commands
          */
	dev->base_addr = NW_IOADDR;

	dev->irq = NW_IRQ;

	/* Fill up the ethernet address; this is normally obtained from a 
                     * some initial boot settings
                     */
       	 FILL_ETHER_ADDRESS(dev->dev_addr);

	/* request IRQ */
	request_irq(dev->irq, lxNW_Isr, 0, "NW", dev);

	RESET_NW();

	/* Fill in the important functions in the device structure */
	dev->open = lxNW_open;
	dev->hard_start_xmit = lxNW_SendPacket;
	dev->stop = lxNW_close;
	dev->watchdog_timeo = HZ;

	/* ether_setup is provided to fill in the default ethernet fields. One important
         	* field here is the transmit queue length maintained per device.
	 * The default is 100. Also the dev->flags set are IFF_BROADCAST and 
         	* IFF_MULTICAST which means that the device supports broadcasting
         	* and has multicast support. In case your device does not support
         	* multicasting, it needs to be explicitly cleared.
         	*/ 
	ether_setup(dev);

	return 0;
}

#ifdef MODULE
static struct net_device dev;

static int init_module(void)
{
	dev.init = lxNWprobe;

	register_netdev(&dev);
	return 0;
}

static void cleanup_module(void)
{
	unregister_netdev(&dev);
}

#endif


static int lxNW_open(struct net_device *dev)
{
	
	RESET_NW(); INIT_NW();

	/* Start the device's transmit queue */
	netif_start_queue(dev);
}


static int lxNW_close(struct net_device *dev)
{
	RESET_NW();

	/* Stop the device's transmit queue */
	netif_stop_queue(dev);
}


static void lxNW_Isr(int irq, void *id, struct pt_regs *regs)
{

	struct net_device *dev = id;

	switch (READ_INTERRUPT_CONDITION())
	{
		case TX_EVENT:
		netif_wake_queue(dev);
		break;

		case RX_EVENT:
		lxNW_Receive(dev);
		break;
	}

}

static int lxNW_SendPacket(struct sk_buff *skb, struct net_device *dev)
{

	/* disable the irq since that can trigger a softirq */
	disable_irq(dev->irq);

	netif_stop_queue(dev);

	FILL_TX_NW(skb);

	enable_irq(dev->irq);

	dev_kfree_skb(skb);

	return 0;
	
}


static void lxNW_Receive(struct net_device *dev)
{
	struct sk_buff *skb;

	/* Allocate the skb after getting the length of the frame from hw */
	skb = dev_alloc_skb(READ_RX_LEN() + 2);

	skb_reserve(skb,2); /* This is done for alignment to 16 bytes */

	skb->dev = dev;
	READ_RX_NW(skb);
	skb->protocol = eth_type_trans(skb,dev);
	netif_rx(skb);
}
