
#include <linux/module.h>
#include <linux/config.h>

#include <linux/types.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/fcntl.h>
#include <linux/fs.h>
#include <linux/init.h>

#include <asm/uaccess.h>
#include <asm/system.h>

#include "kapi.h"

#define KAPI_MINOR 111

int my_kernel_func(int val, char *in_str, char *out_str){
    printk(KERN_DEBUG"val = %d, str = %s\n", val, in_str);
    strcpy(out_str, "Hello User Space");
    return 2;
}


int kapi_open_cnt;

static int
kapi_ioctl(struct inode *inode, struct file *file, unsigned int cmd, unsigned long arg)
{
	int i,err;
    kfunc_t kdata,udata;

    if(copy_from_user(&udata, (kfunc_t *)arg, sizeof(kfunc_t)))
        return -EFAULT;

    for (i = 0 ; i < udata.num ; i++){
        kdata.arg[i].val = kmalloc(udata.arg[i].size, GFP_KERNEL);
        if (udata.arg[i].dir == DIR_IN){
            if (copy_from_user(kdata.arg[i].val, udata.arg[i].val, udata.arg[i].size))
                goto error;
        }
    }
    kdata.ret.val = kmalloc(udata.ret.size, GFP_KERNEL);

	switch (cmd) {

	    case MY_KERNEL_FUNC:
            *(int *)(kdata.ret.val) = my_kernel_func(*(int *)kdata.arg[0].val, (char *)kdata.arg[1].val, (char *)kdata.arg[2].val);
        break;

	    default:
		    return -EINVAL;
	}

    err = 0;
    for (i = 0 ; i < udata.num ; i++){
        if (udata.arg[i].dir == DIR_OUT){
            if (copy_to_user(udata.arg[i].val, kdata.arg[i].val, udata.arg[i].size))
                err = -EFAULT;
        }
        kfree(kdata.arg[i].val);
    }

    if (copy_to_user(udata.ret.val, kdata.ret.val, udata.ret.size))
        err = -EFAULT;

    kfree(kdata.ret.val);

    return err;
    
error:
    {
        int j;
        for (j = 0 ; j <= i; j++)
            kfree(kdata.arg[j].val);
    }

    return -EFAULT;
    
}

static int
kapi_open(struct inode *inode, struct file *file)
{
	kapi_open_cnt++;

	return 0;
}

static int
kapi_release(struct inode *inode, struct file *file)
{

	kapi_open_cnt--;

	return 0;
}


static struct file_operations kapi_fops = {
	.owner		= THIS_MODULE,
	.llseek		= NULL,
	.read		= NULL,
	.write		= NULL,
	.ioctl		= kapi_ioctl,
	.open		= kapi_open,
	.release	= kapi_release,
};

static struct miscdevice kapi_dev = {
	KAPI_MINOR,
	"kapi",
	&kapi_fops,
};

static int __init
kapi_init(void)
{
	int ret;

	ret = misc_register(&kapi_dev);
	if (ret) 
		printk(KERN_ERR "kapi: can't misc_register on minor=%d\n",
		    KAPI_MINOR);
	return ret;

}

static void __exit
kapi_cleanup_module(void)
{
	misc_deregister(&kapi_dev);
}

module_init(kapi_init);
module_exit(kapi_cleanup_module);

MODULE_LICENSE("GPL");

MODULE_ALIAS_MISCDEV(KAPI_MINOR);
