#include <pthread.h>
#include <stdio.h>

#define SUCCESS 0
#define FAIL    1
#define MAX_SIZE 16*1024

extern char *get_queue(void);
extern void add_queue(char *buffer);
extern int is_empty_queue(void);

pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cond = PTHREAD_COND_INITIALIZER;

void* audio_init(void *sample){
     int init_status = SUCCESS;

     printf("Audio init thread created with ID %d\n", pthread_self());

    /* Initialize MP3 decoder subsystem. set init_status = FAIL for failure. */

    /* Initialize Audio output subsystem. set init_status = FAIL for failure */

    if ((int)sample){

        /* Play sample output for 2 second. Set init_status = FAIL if play fails */

    }

    printf("Audio subsystem initialized\n");

    pthread_exit((void *)init_status);
}
    
int system_init(){
    pthread_t audio_tid;
    int sample = 1;
    void * audio_init_status;

    /* Initialize audio subsystem in a separate thread */
    if (pthread_create(&audio_tid, NULL, audio_init, (void *)sample) != 0){
        printf("Audio thread creation failed.\n");
        return FAIL;
    }

    /*
     * Initialize rest of application, data structures etc
     */

    /* Wait for audio_init thread to complete */
    pthread_join(audio_tid, &audio_init_status);

    /* If audio init failed then return error */
    if ((int)audio_init_status == FAIL){
        printf("Audio init failed.\n");
        return FAIL;
    }

    return SUCCESS;
}

void* audio_decoder(void *unused){

   char *buffer; 

   printf("Audio Decoder thread started\n");

   for(;;){
        pthread_mutex_lock(&lock);
    
        while(is_empty_queue())
            pthread_cond_wait(&cond, &lock);   

        buffer = get_queue();

        pthread_mutex_unlock(&lock);

        /* decoder data in buffer */

        /* send to ouput for playback */

        free(buffer);
  }
}

int main(){

   FILE *fp;
   pthread_t decoder_tid;

   system_init();

   fp = fopen("song.mp3", "r");

    /* Create audio decoder thread */
    if (pthread_create(&decoder_tid, NULL, audio_decoder, NULL ) != 0){
        printf("Audio decoder thread creation failed.\n");
        return FAIL;
    }

   while (!feof(fp)){
     char *buffer = (char *)malloc(MAX_SIZE);
     fread(buffer, MAX_SIZE, 1, fp);
     
     pthread_mutex_lock(&lock);
     add_queue(buffer);
     pthread_mutex_unlock(&lock);
     pthread_cond_signal(&cond);

     usleep(300*1000);
   }

   pthread_cancel(decoder_tid);
   pthread_join(decoder_tid, NULL);

}
