#include <stdio.h>
#include <sys/time.h>

/* You can tune this number to support more profiler instances */
#define MAX_INSTANCE 5

/* Used by eproc_curr */
long long int eprof_curr[MAX_INSTANCE] ;
/* Used by eprof_stop */
long long int eprof_diff[MAX_INSTANCE] ;
long eprof_calls[MAX_INSTANCE] ;
/* Instance label */
char * eprof_label[MAX_INSTANCE] ;

/* Returns the current time in microseconds */
static long long  get_curr_time(){
	struct timeval tv;
	gettimeofday(&tv,NULL);
	return tv.tv_sec*1000000 + tv.tv_usec;
}

/* Initialize */
void eprof_init () {
  int i;
  for (i=0; i<MAX_INSTANCE; i++) {
    eprof_diff[i] = 0;
    eprof_curr[i] = 0;
    eprof_calls[i] = 0;
    eprof_label[i] = NULL;
  }
}

/* Allocate an instance */
int eprof_alloc(char *label) {
  int id;
  for (id = 0; id < MAX_INSTANCE && eprof_label[id] != NULL; id++)
    ;

  if (id == MAX_INSTANCE)
    return -1;

  eprof_label[id] = label;
  return id;
}

/* Record the current time */
void eprof_start(int id) {

  if (id >= 0 && id <  MAX_INSTANCE) 
  	eprof_curr[id] = get_curr_time();
}

/* Store the diff from previous call to start */
void eprof_stop(int id) {

  if (id >= 0 && id <  MAX_INSTANCE) {
  	eprof_diff[id] += get_curr_time() - eprof_curr[id];
  	eprof_calls[id]++;
  }

}

/* Free the profiler instance */
void eprof_free(char *label){
  int id;
  for (id = 0; id < MAX_INSTANCE && strcmp(label, eprof_label[id]) != 0; id++)
    ;
  if (id < MAX_INSTANCE)
  	eprof_label[id] = NULL;
}

/* Finally print the result */
void eprof_print () {
  int i;

  printf ("\neProf Result:\n\n"
	  "%s  %.15s  %20s     %10s\n"
	  "---------------------------------------------------\n",
	  "ID", "name",  "calls", "usec/call");
  for (i=0; i<MAX_INSTANCE; i++) {
    if (eprof_label[i]) {
        printf ("%d: %.15s %20d",
	      i, eprof_label[i], eprof_calls[i]);
        if (eprof_calls[i])
	  printf(" %15lld", eprof_diff[i] / eprof_calls[i]);
        printf ("\n");
    }
  }
}

