#include <pthread.h>
#include "eprof.h"

#define MAX_LOOP 10

/* In this function we want to profile complete loop */
void func_1(void * dummy){
	int i;
	/* Allocate a profiler instance */
        int id = eprof_alloc("func1");
	/* Start the profiler */
	eprof_start(id);

	for (i = 0; i < MAX_LOOP; i++){
		usleep(1000*1000);
	}

	/* Stop the profiler */
	eprof_stop(id);
}

/* In this example, we profile each iteration of the loop */
void func_2(void){
	int i;

	/* Allocate the profiler instance */
        int id = eprof_alloc("func2");

	for (i = 0 ; i < MAX_LOOP; i++){
		/* Start the profiler */
		eprof_start(id);

		usleep(500*1000);

		/* Stop the profiler */
		eprof_stop(id);
	}
}

/* The example creates a thread and runs function 'func_1' in the thread.
 * After starting the thread it runs function 'func_2'. eProf is being used 
 * to profile both these functions at the same time
 */
int main(){
	pthread_t thread_id;

	/* Initialize eProf */
	eprof_init();

	/* Create thread that runs function func_1 */
	pthread_create(&thread_id, NULL, func_1, NULL);
	/* Run function func_2 */
	func_2();
	/* Wait for thread to exit */
	pthread_join(thread_id, NULL);
	/* Print the results */
	eprof_print();
	return 0;
}

